<?php

namespace App\Tools;

use App\Key\K;
use EasySwoole\Component\TableManager;
use Swlib\SaberGM;
use Symfony\Component\Yaml\Yaml;

class Proxy
{
    static $error = '';

    public static function getError()
    {
        return static::$error;
    }

    public static function parseUrl($url='',$filePath='')
    {
        $proxy=[];
        switch (true){
            case !!$url:
                try {
                    $data = SaberGM::get($url);
                    $data = $data->getBody()->getContents();
                } catch (\Throwable $exception) {
                    System::log($exception->getMessage(), 'first parse proxy url error');
                }
                try {
                    if (!$data) {
                        //如果没有获取到订阅的信息,有可能是当前的这个订阅不允许使用大陆之外的IP来请求，尝试使用下面的地址来获取一次结果
                        $data = SaberGM::post('http://fast.zhangwei.fun/index/index/curlSubscribe', ['url' => $url], ['timeout' => 8]);
                        $data = $data->getBody()->getContents();
                        System::log($data, 'fast.zhangwei.fun get subscribe data');
                    }
                } catch (\Throwable $exception) {
                    System::log($exception->getMessage(), 'parse proxy url error');
                    return false;
                }
                $proxy = Yaml::parse($data);
                break;
                //如果给了文件地址，那么直接解析文件的地址
            case !!$filePath:
                if (!file_exists($filePath)){
                    System::log('proxy file path is not exit');
                    return  false;
                }
                $proxy=Yaml::parseFile($filePath);
                break;
        }
//        $data = explode(PHP_EOL, $data);
//        $data = array_filter($data);
//        $data = array_map(function ($v) {
//            $v = explode('//', $v);
//            return json_decode(base64_decode($v['1']), true);
//        }, $data);
        if (!$proxy) {
            System::log('no proxy data');
            return false;
        }
        $parseData = [];
        foreach ($proxy['proxies'] as $v) {
            //注意这个key的组成内容，和后面的代理服务器模型返回的结果是一样的，这样方便进行相关的判断
            $key = $v['server']  . $v['port'] . $v['uuid'];
            $parseData[$key] = $v;
        }
//        foreach ($data as $v) {
//            $key = $v['path'] . '_' . $v['add'] . '_' . $v['port'] . '_' . $v['host'];
//            $parseData[$key] = $v;
//        }
        return $parseData;
    }

    public static function getBaseTemplate()
    {
        try {
            $data = SaberGM::get(K::MANAGE_SERVER_ADDR . '/xt/proxy-clash.json', ['timeout' => 2]);
            $data=$data->getBody()->getContents();
            return json_decode($data, true);
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'get proxy template error');
            return false;
        }
    }

    public static function getAllProxy($all = false)
    {
        try {
            if ($all) {
                //返回全部的代理，包含了过期的，或者是已经超过限制了的
                $url = K::MANAGE_SERVER_ADDR_9502 . '/google/subscribe/index?bigemap=1&all=1';
            } else {
                $url = K::MANAGE_SERVER_ADDR_9502 . '/google/subscribe/index?bigemap=1';
            }
            $data = SaberGM::get($url, ['timeout' => 3]);
            $data = $data->getBody()->getContents();
            $data = json_decode(trim($data), true);
            if (!$data) {
                System::log($data, 'decode proxy error');
                return false;
            }
            return $data['data'];
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'get proxy template error');
            return false;
        }
    }

    //在比如测试的过程中，发现当前的服务器被封了，那么要通知主服务器
    public static function noticeForbiddenServer($key = '')
    {
        if (!$key) return;
        try {
            //由于在这是里面只能获取到代理的key值，那么将代理的key值提交给主服务器
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/google/api/proxy', ['code' => '403_key', 'result' => $key], ['timeout' => 3]);
        } catch (\Throwable $exception) {

        }
    }
    public static function startProxy($template, $config, $port)
    {
        //这个端口号一定是需要是数字类型
        $template['mixed-port'] = intval($port);
        $config['name'] = 'bigemap';
        $template['proxies'] = [$config];
        $template['proxy-groups'][0]['proxies'] = [$config['name']];
        $baseConfig = Yaml::dump($template);
        $cachePath = System::config(K::KEY_CACHE_PATH);
        //所有的代理都是通过config+端口号来操作的，停止也是类似，后来也是加入了clash ,这里并没有修改对应配置文件的加载方案
        $testConfig = $cachePath . '/clash/config' . $port . '.yaml';
        file_put_contents($testConfig, $baseConfig);
        if (!Shell::shellKill('clash/config' . $port)) {
            static::$error = '停止旧的程序失败!';
            System::log('stop old proxy failed');
            return false;
        }
        $clashPath = $cachePath . '/clash/clash';
//        echo $v2rayPath . ' -f=' . $testConfig,PHP_EOL;
        //添加 -d 参数是将当前的目录设置为运行的目录，这样才可以读缓存文件。否则无法如果在国内，无法开启
        if (!Shell::shellStart($clashPath . ' -d '.$cachePath.'/clash  -f=' . $testConfig, dirname($clashPath) . '/log' . $port)) {
            static::$error = '启动测试clash失败!';
            System::log('start test proxy failed');
            return false;
        }
        return true;
    }

    public static function getProxyIp($proxy = '')
    {
        $config = System::config();
        $ip = '';
        $httpConfig = ['timeout' => 2];
        if ($proxy) {
            $httpConfig['proxy'] = $proxy;
        } elseif ($config[K::KEY_PROXY_HISTORY_ENABLE] && $config[K::KEY_PROXY_HISTORY]) {
            $httpConfig['proxy'] = $config[K::KEY_PROXY_HISTORY];
        } else {
            System::log('no proxy');
            return $ip;
        }
        try {
            $proxyIp = SaberGM::get('http://google.zhangwei.fun:9502/google/index/ip', $httpConfig);
            $ip = $proxyIp->getBody()->getContents();
            if (!filter_var($ip, FILTER_VALIDATE_IP)) $ip = '';
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'get proxy ip error current proxy ' . $config[K::KEY_PROXY_HISTORY]);
        }
        return $ip;
    }

    public static function checkIsPrimary($ps = '')
    {
        $primary = false;
        foreach (['香港', '狮城', '新加坡', '日本', '韩国', '台湾', '沪台', '沪韩', '沪日'] as $v) {
            if (strpos($ps, $v) !== false) return true;
        }
        return $primary;
    }

    public static function subServer($data = [])
    {
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $cacheData = $configTable->get(K::KEY_SUB_SERVER_LIST);
        $cacheData['data'] = json_decode($cacheData['data'], true);
        if (!$cacheData['data']) $cacheData['data'] = [];
        if ($data) {
            $key = $data['host'] . '_' . $data['port'];
//            $has=false;
            $data['time'] = time();
            $cacheData['data'][$key] = $data;
            //这里保证在前台选择的时候，顺序不会出错，保存每个服务器都平均的被选择
            ksort($cacheData['data']);
            $cacheData['data'] = array_filter($cacheData['data'], function ($v) {
                //只会返回20秒之内有提交的服务器
                return time() - $v['time'] <= 20;
            });
            $cacheData['data'] = json_encode($cacheData['data']);
            $configTable->set(K::KEY_SUB_SERVER_LIST, $cacheData);
        } else {
            $server = array_filter($cacheData['data'], function ($v) {
                //只会返回20秒之内有提交的服务器
                return time() - $v['time'] <= 20;
            });
            return $server;
        }
    }

    //如果子服务器因为某种原因停止了，那么马上会删除这个子服务器
    public static function removeSubServer($port = '')
    {
        if (!$port) return;
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $cacheData = $configTable->get(K::KEY_SUB_SERVER_LIST);
        $cacheData['data'] = json_decode($cacheData['data'], true);
        if (!$cacheData['data']) $cacheData['data'] = [];
        foreach ($cacheData['data'] as $k => $v) {
            if ($v['port'] == $port) {
                //直接通过端口号来删除
                unset($cacheData['data'][$k]);
                break;
            }
        }
        $cacheData['data'] = json_encode($cacheData['data']);
        $configTable->set(K::KEY_SUB_SERVER_LIST, $cacheData);

    }
}
